'===============================================================================
' Copyright  Micromega Corporation 2004-2005. All rights reserved
'
' @file	sample-i2c.bas
' @target	PICAXE
'
' Demonstration routine for uM-FPU V2 using an I2C interface.
' The demo program takes an integer value representing the diameter
' of a circle in centimeters, converts the value to inches, then calculates
' the circumference and area in inches and square inches.
'
' @author Cam Thompson, Micromega Corporation
' @version V2.0 - January 20, 2005
' changes:
'	V2.0 - January 20, 2005
'	- updated for uM-FPU V2.0
'	V0.1 - July 5, 2004
'	- original version
'===============================================================================

'===============================================================================
'-------------------- uM-FPU V2.0 PICAXE definitions ---------------------------
'===============================================================================

symbol	fpuID		= 0xC8	' uM-FPU I2C address

'-------------------- uM-FPU opcodes -------------------------------------------

symbol	SELECTA	= 0x00	' select A register
symbol	SELECTB	= 0x10	' select B register
symbol	FWRITEA	= 0x20	' write to register and select A
symbol	FWRITEB	= 0x30	' write to register and select B
symbol	FREAD		= 0x40	' read from register
symbol	FSET		= 0x50	' A = REG
symbol	LSET		= 0x50	' A = REG


symbol	FADD		= 0x60	' A = A + REG (float)
symbol	FSUB		= 0x70	' A = A - REG (float)
symbol	FMUL		= 0x80	' A = A * REG (float)
symbol	FDIV		= 0x90	' A = A / REG (float)

symbol	LADD		= 0xA0	' A = A + REG (long)
symbol	LSUB		= 0xB0	' A = A - REG (long)
symbol	LMUL		= 0xC0	' A = A * REG (long)
symbol	LDIV		= 0xD0	' A = A / REG (long)

symbol	SQRT		= 0xE0	' A = sqrt(A)	
symbol	LOG		= 0xE1	' A = ln(A)
symbol	LOG10		= 0xE2	' A = log(A)
symbol	EXP		= 0xE3	' A = e ** A
symbol	EXP10		= 0xE4	' A = 10 ** A
symbol	SIN		= 0xE5	' A = sin(A) radians
symbol	COS		= 0xE6	' A = cos(A) radians
symbol	TAN		= 0xE7	' A = tan(A) radians
symbol	FLOOR		= 0xE8	' A = nearest integer <= A
symbol	CEIL		= 0xE9	' A = nearest integer >= A
symbol	ROUND		= 0xEA	' A = nearest integer to A
symbol	NEGATE	= 0xEB	' A = -A
symbol	ABS		= 0xEC	' A = |A|
symbol	INVERSE	= 0xED	' A = 1 / A
symbol	DEGREES	= 0xEE	' A = A / (PI / 180) radians to degrees
symbol	RADIANS	= 0xEF	' A = A * (PI / 180) degreees to radians
symbol	SYNC		= 0xF0	' synchronization
symbol	FLOAT		= 0xF1	' copy A to register 0 and convert to float
symbol	FIX		= 0xF2	' copy A to register 0 and convert to long
symbol	FCOMPARE	= 0xF3	' compare A and B
symbol	LOADBYTE	= 0xF4	' write signed byte to reg 0, convert to float
symbol	LOADUBYTE	= 0xF5	' write unsigned byte to reg 0, convert to float
symbol	LOADWORD	= 0xF6	' write signed word to reg 0, convert to float
symbol	LOADUWORD	= 0xF7	' write unsigned word to reg 0, convert to float
symbol	READSTR	= 0xF8	' read zero terminated string
symbol	ATOF		= 0xF9	' convert ASCII to float, store in A register
symbol	FTOA		= 0xFA	' convert float to ASCII
symbol	ATOL		= 0xFB	' convert ASCII to long, store in A register
symbol	LTOA		= 0xFC	' convert long to ASCII
symbol	FSTATUS	= 0xFD	' get the status of A register
symbol	XOP		= 0xFE	' extended opcode
symbol	NOP		= 0xFF	' nop

symbol	FUNCTION	= 0x00	' (XOP) user functions 0-15
symbol	READBYTE	= 0x90      ' (XOP) read low 8 bits of A (long)
symbol	READWORD	= 0x91      ' (XOP) read low 16 bits of A (long)
symbol	READLONG	= 0x92      ' (XOP) read 32-bit long value from A
symbol	READFLOAT	= 0x93      ' (XOP) read floating point value from A
symbol	LINCA		= 0x94      ' (XOP) A = A + 1 (long)
symbol	LINCB		= 0x95      ' (XOP) A = A + 1 (long)
symbol	LDECA		= 0x96      ' (XOP) A = A - 1 (long)
symbol	LDECB		= 0x97      ' (XOP) A = A - 1 (long)
symbol	LAND		= 0x98      ' (XOP) A = A AND B (long)
symbol	LOR		= 0x99      ' (XOP) A = A OR B (long)
symbol	LXOR		= 0x9A      ' (XOP) A = A XOR B (long)
symbol	LNOT		= 0x9B      ' (XOP) A = NOT A (long)
symbol	LTST		= 0x9C      ' (XOP) status of A AND B (long)
symbol	LSHIFT	= 0x9D      ' (XOP) shift A by B bits (long)
symbol	LWRITEA	= 0xA0	' (XOP) write long to register and select A
symbol	LWRITEB	= 0xB0	' (XOP) write long to register and select B
symbol	LREAD		= 0xC0	' (XOP) read long from register
symbol	LUDIV		= 0xD0	' (XOP) A = A / REG (unsigned long)
symbol	POWER		= 0xE0	' (XOP) A = A ** B
symbol	ROOT		= 0xE1	' (XOP) A = the Bth root of A
symbol	FMIN		= 0xE2	' (XOP) A = minimum of A and B
symbol	FMAX		= 0xE3	' (XOP) A = maximum of A and B
symbol	FRACTION	= 0xE4	' (XOP) load reg 0 with the fractional part of A
symbol	ASIN		= 0xE5	' (XOP) A = asin(A) radians
symbol	ACOS		= 0xE6	' (XOP) A = acos(A) radians
symbol	ATAN		= 0xE7	' (XOP) A = atan(A) radians
symbol	ATAN2		= 0xE8	' (XOP) A = atan(A/B)
symbol	LCOMPARE	= 0xE9	' (XOP) long compare A and B
symbol	LUCOMPARE	= 0xEA	' (XOP) unsigned long compare A and B
symbol	LSTATUS	= 0xEB	' (XOP) long status
symbol	LNEGATE	= 0xEC	' (XOP) A = -A (long)
symbol	LABS		= 0xED	' (XOP) A = |A| (long)
symbol	LEFT		= 0xEE	' (XOP) right parenthesis
symbol	RIGHT		= 0xEF	' (XOP) left parenthesis
symbol	LOADZERO	= 0xF0	' (XOP) load register 0 with zero
symbol	LOADONE	= 0xF1	' (XOP) load register 0 with 1.0
symbol	LOADE		= 0xF2	' (XOP) load register 0 with e
symbol	LOADPI	= 0xF3	' (XOP) load register 0 with pi
symbol	LONGBYTE	= 0xF4	' (XOP) write signed byte to reg 0, convert to long
symbol	LONGUBYTE	= 0xF5	' (XOP) write unsigned byte to reg 0, convert to long
symbol	LONGWORD	= 0xF6	' (XOP) write signed word to reg 0, convert to long
symbol	LONGUWORD	= 0xF7	' (XOP) write unsigned word to reg 0, convert to long
symbol	IEEEMODE	= 0xF8	' (XOP) set IEEE mode (default)
symbol	PICMODE	= 0xF9	' (XOP) set PIC mode
symbol	CHECKSUM	= 0xFA	' (XOP) calculate code checksum
symbol	BREAK		= 0xFB	' (XOP) debug breakpoint
symbol	TRACEOFF	= 0xFC	' (XOP) turn debug trace off
symbol	TRACEON	= 0xFD	' (XOP) turn debug trace on
symbol	TRACESTR	= 0xFE	' (XOP) send debug string to trace buffer
symbol	VERSION	= 0xFF	' (XOP) get version string

symbol	syncChar	= 0x5C      ' sync character 

'-------------------- uM-FPU variables -----------------------------------------

symbol	dataWord	= w6		' (b13,b12) data word
symbol	dataHigh	= b13		' high byte of dataWord
symbol	dataLow	= b12		' low byte of dataWord
symbol	dataByte	= b12		' (alternate name)
symbol	opcode	= b13		' opcode (same as dataHigh)
symbol	opcode2	= b12		' opcode2 (same as dataLow)
symbol	reg		= b13		' register value (same as dataHigh)
symbol	format	= b12		' format (same as dataLow)
symbol	fpu_status	= b12		' status byte (same as dataLow)

'-------------------- uM-FPU status bits ---------------------------------------

symbol	IS_ZERO	= 0x81	' positive zero
symbol	IS_NZERO	= 0x83	' negative zero
symbol	IS_NEGATIVE	= 0x82	' negative
symbol	IS_NAN	= 0x84	' NaN (Not-a-Number)
symbol	IS_PINF	= 0x88	' positive infinity
symbol	IS_NINF	= 0x8A	' negative infinity

'==================== end of uM-FPU definitions ================================

'===============================================================================
'==================== main definitions =========================================
'===============================================================================

'-------------------- constants ------------------------------------------------

symbol	DiameterIn	  = 1       ' dimeter in inches  (uM-FPU register)
symbol	Circumference = 2       ' circumference      (uM-FPU register)
symbol	Area		  = 3       ' area               (uM-FPU register)
symbol	Pi		  = 4       ' constant pi        (uM-FPU register)
symbol	F2_0		  = 5       ' constant 2.0       (uM-FPU register)
symbol	F2_54		  = 6       ' constant 2.54      (uM-FPU register)

'-------------------- variables ------------------------------------------------

symbol	diameterCm	  = W0      ' diameter in centimeters

'===============================================================================
'-------------------- initialization -------------------------------------------
'===============================================================================

reset:
	sertxd(13, 10, "Conversion Example", 13, 10)

	i2cslave fpuID, i2cfast, i2cbyte
	gosub fpu_reset			' reset the FPU hardware
	if fpu_status = SyncChar then reset2
	sertxd (13, 10, "uM-FPU not detected.")
	end

reset2:
	gosub print_version		' display the uM-FPU version number

	'-----------------------------------------------------
	' Load constants for later use.
	' Three different ways of loading constants are shown.
	'-----------------------------------------------------
	writei2c 0, (Pi, XOP, LOADPI, FSET)
	writei2c 0, (F2_0, LOADBYTE, 2, FSET)
	writei2c 0, (F2_54, ATOF, "2.54", 0, FSET)

'===============================================================================
'-------------------- main routine ---------------------------------------------
'===============================================================================

main:
	'---------------------------------------------------------------------------
	' Get diameter in centimeters.  The value would typically come from a sensor
	' reading, but in this example an assumed value of 25 is used.
	'---------------------------------------------------------------------------
	diameterCm = 25
	sertxd(13, 10, "Diameter (cm): ", #diameterCm)

	'-------------------------------
	' diameterIn = diameterCm / 2.54
	'-------------------------------
	opcode = FDIV+F2_54
	writei2c 0, (DiameterIn, LOADBYTE, diameterCm, FSET, opcode)
	sertxd(13, 10, "Diameter (in.): ")
	gosub Print_Float

	'------------------------------
	' circumference = diameter * pi
	'------------------------------
	opcode = FSET+DiameterIn
	opcode2 = FMUL+Pi
	writei2c 0, (Circumference, opcode, opcode2)
	sertxd(13, 10, "Circumference (in.): ")
	gosub Print_Float

	'----------------------------
	'area = (diameter / 2)^2 * pi
	'----------------------------
	opcode = FSET+DiameterIn
	opcode2 = FDIV+F2_0
	writei2c 0, (Area, opcode, opcode2)
	opcode = FMUL+Area
	opcode2 = FMUL+Pi
	writei2c 0, (opcode, opcode2)
	sertxd(13, 10, "Area (sq.cm.): ")
	gosub Print_Float

	sertxd(13, 10, 13, 10, "Done.") 'end of program
	end

'===============================================================================
'-------------------- uM-FPU V2.0 PICAXE support routines ----------------------
'===============================================================================

fpu_reset:
	writei2c 1, (0)			' reset the uM-FPU
	pause 8				' wait for reset to complete
	
	writei2c 0, (SYNC)		' check for synchronization
						' (fall through to fpu_readStatus)
fpu_readStatus:
	readi2c 0, (fpu_status)		' read status byte
	return

fpu_wait:
	readi2c 0, (fpu_status)		' wait for ready status
	if fpu_status <> 0 then fpu_wait
	return

print_version:
	writei2c 0, (XOP, VERSION)	' get the uM-FPU version string
	goto print_string2            ' print it

print_float:
	format = 0				' set format to zero (free format)
						' (fall through to print_floatFormat)
print_floatFormat:
	opcode = FTOA			' convert floating point to formatted ASCII
	goto print_string			' print the value

print_long:
	format = 0				' set format to 0 (free format)
						' (fall through to print_longFormat)
print_longFormat:
	opcode = LTOA			' convert long integer to formatted ASCII
						' (fall through to print_string)
print_string:
	writei2c 0, (opcode, format)	' send conversion command

print_string2:
	gosub fpu_wait			' wait until uM-FPU is ready
	writei2c 0, (READSTR)

print_string3:		
	readi2c 0, (dataByte)		' display zero terminated string
	if dataByte = 0 then Print_String4
	if dataByte > 127 then Print_String4
	sertxd(dataByte)
	goto Print_String3

print_string4:
	return				' end of string

'==================== end of uM-FPU support routines ===========================

